const { S3Client, CreateMultipartUploadCommand, UploadPartCommand, CompleteMultipartUploadCommand, PutObjectCommand, DeleteObjectCommand } = require('@aws-sdk/client-s3');
const fs = require('fs');
const { v4: uuidv4 } = require('uuid');
const path = require('path');
// const sharp = require('sharp');
// const ffmpeg = require('fluent-ffmpeg');
const { AWS_BUCKET_NAME, AWS_BUCKET_REGION, AWS_PUBLIC_KEY, AWS_SECRET_KEY } = require('./index.js');

const client = new S3Client({
  region: AWS_BUCKET_REGION,
  credentials: {
    accessKeyId: AWS_PUBLIC_KEY,
    secretAccessKey: AWS_SECRET_KEY
  },
  maxAttempts: 5 // Retry strategy
});

// const optimizeImage = async (filePath) => {
//   return sharp(filePath)
//     .resize({ width: 1024 })
//     .toBuffer();
// };

const buildFileUrl = (folder, fileName) => {
  return `https://${AWS_BUCKET_NAME}.s3.${AWS_BUCKET_REGION}.amazonaws.com/${folder}/${fileName}`;
};

const buildFileUri = (path) => {
  return `https://${AWS_BUCKET_NAME}.s3.${AWS_BUCKET_REGION}.amazonaws.com/${path}`;
};

// const optimizeVideo = (filePath) => {
//   return new Promise((resolve, reject) => {
//     const optimizedFilePath = `${filePath}-optimized.mp4`;

//     ffmpeg(filePath)
//       .outputOptions('-vf', 'scale=1280:-2')
//       .output(optimizedFilePath)
//       .on('end', () => resolve(optimizedFilePath))
//       .on('error', (err) => reject(err))
//       .run();
//   });
// };

const uploadMultipart = async (file, folder) => {
  if (!file) {
    return null;
  }

  const filePath = file.tempFilePath;
  const fileSize = fs.statSync(filePath).size;
  const uniqueFileName = `${uuidv4()}_${file.name}`;
  const key = path.join(folder, uniqueFileName);

  const createMultipartUpload = new CreateMultipartUploadCommand({
    Bucket: AWS_BUCKET_NAME,
    Key: key
  });

  const multipartUpload = await client.send(createMultipartUpload);
  const uploadId = multipartUpload.UploadId;

  const partSize = 10 * 1024 * 1024; // 10MB part size
  const parts = [];
  let partNumber = 1;
  let uploadedBytes = 0;

  while (uploadedBytes < fileSize) {
    const chunk = fs.createReadStream(filePath, {
      start: uploadedBytes,
      end: Math.min(uploadedBytes + partSize, fileSize) - 1
    });

    const uploadPart = new UploadPartCommand({
      Bucket: AWS_BUCKET_NAME,
      Key: key,
      PartNumber: partNumber,
      UploadId: uploadId,
      Body: chunk
    });

    const uploadedPart = await client.send(uploadPart);
    parts.push({
      ETag: uploadedPart.ETag,
      PartNumber: partNumber
    });

    uploadedBytes += partSize;
    partNumber += 1;
  }

  const completeMultipartUpload = new CompleteMultipartUploadCommand({
    Bucket: AWS_BUCKET_NAME,
    Key: key,
    UploadId: uploadId,
    MultipartUpload: {
      Parts: parts
    }
  });

  await client.send(completeMultipartUpload);

  const fileUrl = buildFileUrl(folder, uniqueFileName);

  return {
    fileUrl,
    fileName: uniqueFileName,
    folder,
    originalName: file.name,
    size: file.size,
    mimetype: file.name.split('.').pop(),
    encoding: file.encoding,
    path: `${folder}/${uniqueFileName}`
  };
};

const uploadFile = async (file, folder) => {
  if (!file) {
    return null;
  }

  console.log('File object:', file); // Agrega esta línea para depuración

  // Validar que el mimetype esté definido antes de proceder
  if (!file.mimetype) {
    console.error('File mimetype is not defined for file:', file);
    throw new Error('File mimetype is not defined');
  }

  try {
    const filePath = file.tempFilePath;
    // let body;

    if (file.size > 100 * 1024 * 1024) { // Check if file is larger than 100MB
      return await uploadMultipart(file, folder);
    }

    // if (file.mimetype.startsWith('image/')) {
    //   const optimizedImageBuffer = await optimizeImage(filePath);
    //   body = optimizedImageBuffer;
    // } else
    // if (file.mimetype.startsWith('video/')) {
    //   const optimizedFilePath = await optimizeVideo(filePath);
    //   body = fs.createReadStream(optimizedFilePath);
    // } else {
    // }
    const body = fs.createReadStream(filePath);

    const uniqueFileName = `${uuidv4()}_${file.name.replace(/ /g, '_')}`;
    const uploadParams = {
      Bucket: AWS_BUCKET_NAME,
      Key: path.join(folder, uniqueFileName),
      Body: body
    };

    await client.send(new PutObjectCommand(uploadParams));
    const fileUrl = buildFileUrl(folder, uniqueFileName);

    return {
      fileUrl,
      fileName: uniqueFileName,
      folder,
      originalName: file.name,
      size: file.size,
      mimetype: file.name.split('.').pop(),
      encoding: file.encoding,
      path: `${folder}/${uniqueFileName}`
    };
  } catch (error) {
    console.error('Error uploading file:', error);
    throw new Error('Error uploading file');
  }
};

const deleteFileS3 = async (filePath) => {
  const params = {
    Bucket: AWS_BUCKET_NAME,
    Key: filePath

  };

  try {
    await client.send(new DeleteObjectCommand(params));
    // console.log('Archivo eliminado exitosamente', data);
  } catch (err) {
    console.error('Error al eliminar el archivo', err);
  }
};

module.exports = {
  uploadFile,
  uploadMultipart,
  deleteFileS3,
  buildFileUri
};
