const { default: mongoose } = require("mongoose");
const Epocas = require("../models/Epocas");
const Ventas = require("../models/Ventas");
const Despachos = require("../models/Despachos");
const Usuarios = require("../models/Usuarios");

const getEstadisticas = async (req, res) => {
  try {
    //Paquetes mas vendidos
    const resultPaquetes = Ventas.aggregate([
      {
        $match: {
          producto: {
            $ne: mongoose.Types.ObjectId("66da43e02412e32ff68279d9"),
          },
        },
      },
      {
        $group: {
          _id: "$producto",
          totalVentas: { $sum: 1 },
        },
      },
      {
        $sort: { totalVentas: -1 },
      },
      {
        $limit: 5,
      },
      {
        $lookup: {
          from: "productos",
          localField: "_id",
          foreignField: "_id",
          as: "productoDetalles",
        },
      },
      {
        $unwind: "$productoDetalles",
      },
      {
        $project: {
          _id: 0,
          cantidad: "$totalVentas",
          tipo_producto: "$productoDetalles.tipo_producto",
          nombre: "$productoDetalles.nombre",
        },
      },
    ]);

    //Estados con mas despachos
    const resultEstados = Despachos.aggregate([
      {
        $match: { estatus: "Activo" },
      },
      {
        $group: {
          _id: "$estado",
          totalDespachos: { $sum: 1 },
        },
      },
      {
        $sort: { totalDespachos: -1 },
      },
      {
        $limit: 5,
      },
      {
        $lookup: {
          from: "estados",
          localField: "_id",
          foreignField: "_id",
          as: "estadoDetalles",
        },
      },
      {
        $unwind: "$estadoDetalles",
      },
      {
        $project: {
          _id: 0,
          nombre: "$estadoDetalles.nombre",
          cantidad: "$totalDespachos",
        },
      },
    ]);

    //Distribuidores con mas ventas

    const resultDistribuidoresVentas = Ventas.aggregate([
      {
        $match: { "distribuidor.distribuidor": { $ne: null } },
      },
      {
        $group: {
          _id: "$distribuidor.distribuidor",
          totalVentas: { $sum: 1 },
          totalImporte: { $sum: "$importe" },
        },
      },
      {
        $sort: { totalVentas: -1 },
      },
      {
        $lookup: {
          from: "usuarios",
          localField: "_id",
          foreignField: "_id",
          as: "distribuidorDetalles",
        },
      },
      {
        $unwind: "$distribuidorDetalles",
      },
      {
        $project: {
          _id: 0,
          nombre: "$distribuidorDetalles.nombre",
          cantidad: "$totalVentas",
          //   totalImporte: "$totalImporte",
        },
      },
      {
        $limit: 5,
      },
    ]);

    //Meses con mas ventas
    const months = [
      "Enero",
      "Febrero",
      "Marzo",
      "Abril",
      "Mayo",
      "Junio",
      "Julio",
      "Agosto",
      "Septiembre",
      "Octubre",
      "Noviembre",
      "Diciembre",
    ];

    const anioActual = new Date().getFullYear();
    const mesesConMasVentas = Ventas.aggregate([
      {
        $match: {
          $expr: {
            $eq: [{ $year: "$fechaVenta" }, anioActual],
          },
        },
      },
      {
        $group: {
          _id: {
            mes: { $month: "$fechaVenta" },
            anio: { $year: "$fechaVenta" },
          },
          totalVentas: { $sum: 1 },
          totalImporte: { $sum: "$importe" },
        },
      },
      {
        $sort: { totalVentas: -1 },
      },
      {
        $project: {
          _id: 0,
          mes: "$_id.mes",
          anio: "$_id.anio",
          cantidad: "$totalVentas",
          totalImporte: "$totalImporte",
        },
      },
      {
        $limit: 5,
      },
    ]);

    //Contadores

    const despachosActivosCount = Usuarios.countDocuments({
      tipo: "despacho",
      estatus: "Activo",
    });

    const distribuidoresActivosCount = Usuarios.countDocuments({
      tipo: "distribuidor",
      estatus: "Activo",
    });

    const despachosCount = Despachos.countDocuments({
      estatus: "Activo",
    });

    const [
      paquetesVendidos,
      estadosDespacho,
      distribuidores,
      meses,
      abogados,
      distribuidoresCount,
      despachos,
    ] = await Promise.all([
      resultPaquetes,
      resultEstados,
      resultDistribuidoresVentas,
      mesesConMasVentas,
      despachosActivosCount,
      distribuidoresActivosCount,
      despachosCount,
    ]);

    const mesesVentas = meses.map((venta) => ({
      nombre: months[venta.mes - 1],
      anio: venta.anio,
      total: venta.cantidad,
      cantidad: venta.totalImporte,
    }));

    const data = {
      paquetesVendidos,
      estadosDespacho,
      distribuidores,
      mesesVentas,
      abogados,
      distribuidoresCount,
      despachos,
    };
    return res.status(200).json(data);
  } catch (error) {
    console.log(error);
    return res.status(500).json({
      mensaje: "Error al obtener las estadísticas",
      error,
    });
  }
};

module.exports = {
  getEstadisticas,
};
