const { Schema, model } = require('mongoose');
const mongoosePaginate = require('mongoose-paginate-v2');

const UsuariosSchema = new Schema(
  {
    ultimoAcceso: {
      type: Date,
      required: false,
      default: null
    },
    clave: {
      // solo para distribuidores
      type: String,
      required: false,
      index: true
    },
    nombre: {
      type: String,
      required: true,
      trim: true,
      index: true
    },
    fechaNacimiento: {
      type: Date,
      required: false,
      default: null
    },
    apellidoPaterno: {
      type: String,
      required: false,
      default: '',
      trim: true
    },
    apellidoMaterno: {
      type: String,
      required: false,
      default: '',
      trim: true
    },
    telefono: {
      type: String,
      required: false,
      default: '',
      trim: true
    },
    email: {
      type: String,
      required: true,
      trim: true,
      index: true
    },
    password: {
      type: String,
      required: true
    },
    foto: {
      type: String,
      required: false,
      default: ''
    },
    tipoUsuario: {
      type: Schema.Types.ObjectId,
      ref: 'tipoUsuarios',
      required: false,
      index: true
    },
    tipo: {
      type: String,
      required: true,
      enum: ['admin', 'despacho', 'distribuidor'],
      default: 'admin',
      index: true
    },
    token: {
      type: String,
      required: false,
      default: ''
    },
    despacho: {
      type: Schema.Types.ObjectId,
      ref: 'despachos',
      required: false,
      index: true
    },
    tour: {
      type: Boolean,
      required: false,
      default: false,
      index: true
    },
    estatus: {
      type: String,
      enum: ['Activo', 'Inactivo', 'Prospecto'],
      required: true,
      default: 'Activo',
      index: true
    },
    licencia: {
      // solo para distribuidores actual corriendo
      producto: {
        type: Schema.Types.ObjectId,
        ref: 'productos',
        required: false,
        default: undefined
      },
      vigencia: {
        type: Date,
        required: false,
        default: undefined
      },
      referidoPor: {
        type: Schema.Types.ObjectId,
        ref: 'usuarios',
        required: false,
        default: undefined
      },
      comision: {
        type: Number,
        required: false,
        default: 0
      }
    },
    rfc: {
      type: String,
      required: false
    },
    comprobantes: {
      constancia: {
        type: String,
        required: false
      },
      ine: {
        frontal: {
          type: String,
          required: false
        },
        trasera: {
          type: String,
          required: false
        }
      },
      domicilio: {
        type: String,
        required: false
      }
    },
    fechaCreacion: {
      type: Date,
      default: new Date(),
      required: true
    }
  },
  {
    versionKey: false
  }
);

// Pre-save hook to generate distributor key
UsuariosSchema.pre('save', async function (next) {
  if (
    this.tipo === 'distribuidor' &&
    !this.clave &&
    this.estatus === 'Activo'
  ) {
    try {
      // Obtener el último distribuidor con el formato adecuado
      const lastDistribuidor = await this.constructor
        .findOne({
          clave: { $regex: /^DIS-IUS-(\d{4})$/ }
        })
        .sort({ clave: -1 });

      let nextNumber = 1;
      if (lastDistribuidor) {
        const lastNumber = parseInt(lastDistribuidor.clave.split('-')[2], 10);
        nextNumber = lastNumber + 1;
      }

      // Formatear el número con ceros a la izquierda
      const formattedNumber = nextNumber.toString().padStart(4, '0');
      this.clave = `DIS-IUS-${formattedNumber}`;
    } catch (error) {
      return next(error);
    }
  }
  next();
});

UsuariosSchema.plugin(mongoosePaginate);
module.exports = model('usuarios', UsuariosSchema);
